@@ -9,7 +9,6 @@ import torch
 
 import numpy as np
 import pandas as pd
-
 import thingsvision.vision as vision
 
 from collections import defaultdict
@@ -19,6 +18,26 @@ from PIL import Image
 from torchvision import transforms as T
 from typing import Tuple, List, Dict, Iterator, Any
 
+def get_syn2cls_mapping(root:str) -> Dict[str, int]:
+    try:
+        with open(os.path.join(root, 'LOC_synset_mapping.txt'), 'r') as f:
+            cls2syn = dict(enumerate(list(map(lambda x: x.split()[0], f.readlines()))))
+    except FileNotFoundError:
+        raise Exception('\nCould not find LOC_synset_mapping.txt in root directory. Move file to root directory.\n')
+    syn2cls = {syn:cls for cls, syn in cls2syn.items()}
+    return syn2cls
+
+def get_img2cls_mapping(root:str) -> Dict[str, int]:
+    split = root.split('/')[-1] if root.split('/')[-1] else root.split('/')[-2]
+    try:
+        solution = pd.read_csv(os.path.join(root, f'LOC_{split}_solution.csv'))
+    except FileNotFoundError:
+        raise Exception(f'\nCould not find LOC_{split}_solution.csv in root directory. Move file to root directory.\n')
+    syn2cls = get_syn2cls_mapping(root)
+    img2cls = {row[0]+'.JPEG': syn2cls[row[1].split()[0]] for idx, row in solution.iterrows()}
+    img2cls = dict(sorted(img2cls.items(), key=lambda kv:kv[0]))
+    return img2cls, list(syn2cls.keys())
+
 def get_classes(file_names:List[str]) -> Tuple[List[str], Dict[str, list]]:
     cls_to_files = defaultdict(list)
     classes = []
@@ -35,58 +54,76 @@ def get_classes(file_names:List[str]) -> Tuple[List[str], Dict[str, list]]:
     return classes, cls_to_files
 
 def parse_img_name(img_name:str) -> bool:
-    return re.search(r'(.eps|.jpg|.jpeg|.png|.PNG|.tif|.tiff)$', img_name)
+    return re.search(r'(.eps|.jpg|.JPG|.jpeg|.JPEG|.png|.PNG|.tif|.tiff)$', img_name)
 
 def rm_suffix(img:str) -> str:
-    return re.sub(r'(.eps|.jpg|.jpeg|.png|.PNG|.tif|.tiff)$', '', img)
+    return re.sub(r'(.eps|.jpg|.JPG|.jpeg|.JPEG|.png|.PNG|.tif|.tiff)$', '', img)
 
-def instance_dataset(root:str, out_path:str, images:list) -> List[Tuple[str, int]]:
+def instance_dataset(root:str, out_path:str, images:list, imagenet:bool=False, img2cls:dict=None) -> List[Tuple[str, int]]:
     instances = []
     with open(os.path.join(out_path, 'file_names.txt'), 'w') as f:
         for img in images:
             f.write(f'{img}\n')
             instances.append(os.path.join(root, img))
-    samples = tuple((instance, target) for target, instance in enumerate(instances))
+    if imagenet:
+        assert isinstance(img2cls, dict), '\nImage-to-cls mapping must be provided\n'
+        samples = tuple((instance, img2cls[instance.split('/')[-1]]) for instance in instances)
+    else:
+        samples = tuple((instance, target) for target, instance in enumerate(instances))
     return samples
 
-def class_dataset(PATH:str, out_path:str, cls_to_idx:Dict[str, int], things:bool=None, add_ref_imgs:bool=None, cls_to_files:Dict[str, list]=None) -> List[Tuple[str, int]]:
+def class_dataset(
+                  PATH:str,
+                  out_path:str,
+                  cls_to_idx:Dict[str, int],
+                  imagenet:bool=None,
+                  things:bool=None,
+                  add_ref_imgs:bool=None,
+                  cls_to_files:Dict[str, list]=None,
+                  ) -> List[Tuple[str, int]]:
     samples = []
     with open(os.path.join(out_path, 'file_names.txt'), 'w') as f:
-        for target_cls in sorted(cls_to_idx.keys()):
+        classes = cls_to_idx.keys() if (things or imagenet) else sorted(cls_to_idx.keys())
+        for target_cls in classes:
             cls_idx = cls_to_idx[target_cls]
             target_dir = os.path.join(PATH, target_cls)
             if os.path.isdir(target_dir):
                 if cls_to_files is None:
                     for root, _, files in sorted(os.walk(target_dir, followlinks=True)):
                         if (things and add_ref_imgs):
-                            first_img = files[0].rstrip('.jpg')
-                            if not first_img.endswith('b'):
-                                ref_img_path = get_ref_img(first_img)
-                                item = ref_img_path, class_idx
-                                instances.append(item)
-                        for k, file in enumerate(sorted(files)):
+                            first_img = sorted(files)[0]
+                            if parse_img_name(first_img):
+                                first_img = first_img.rstrip('.jpg')
+                                if not first_img.endswith('b'):
+                                    ref_img_path = get_ref_img(first_img)
+                                    item = ref_img_path, cls_idx
+                                    samples.append(item)
+                                    f.write(f'{ref_img_path}\n')
+                            else:
+                                raise Exception(f'\nFound file that does not seem to be in the correct format: {first_img}.\nRemove file before proceeding with feature extraction.\n')
+                        for file in sorted(files):
                             path = os.path.join(root, file)
-                            f.write(f'{path}\n')
                             if os.path.isfile(path) and parse_img_name(file):
                                 item = path, cls_idx
                                 samples.append(item)
+                                f.write(f'{path}\n')
                 else:
                     for f_name in cls_to_files[target_cls]:
                         path = os.path.join(target_dir, f_name)
-                        f.write(f'{path}\n')
                         if os.path.isfile(path) and parse_img_name(f_name):
                             item = path, cls_idx
                             samples.append(item)
+                            f.write(f'{path}\n')
     return samples
 
-def get_ref_img(first_img:str, folder:str='./reference_images/') -> str:
+def get_ref_img(first_img:str, folder:str='./reference_images/', suffix:str='.jpg') -> str:
     if not os.path.exists(folder):
         raise FileNotFoundError(f'Directory for reference images not found. Move reference images to {folder}.')
-    ref_images = [f.name for f in os.scandir(folder) if f.is_file() and parse_img_name(f)]
+    ref_images = [f.name for f in os.scandir(folder) if f.is_file() and parse_img_name(f.name)]
     for ref_img in ref_images:
-        img_name = ref_img.rstrip('.jpg')
+        img_name = ref_img.rstrip(suffix)
         if re.search(f'^{img_name}', first_img):
-            return os.path.join(folder, img_name)
+            return os.path.join(folder, ref_img)
 
 class ImageDataset(object):
     """
@@ -110,6 +147,8 @@ class ImageDataset(object):
                 self,
                 root:str,
                 out_path:str,
+                imagenet_train:bool,
+                imagenet_val:bool,
                 things:bool,
                 things_behavior:bool,
                 add_ref_imgs:bool,
@@ -117,6 +156,8 @@ class ImageDataset(object):
                 transforms=None,
                 ):
         self.root = root
+        self.imagenet_train = imagenet_train
+        self.imagenet_val = imagenet_val
         self.things = things
         self.things_behavior = things_behavior
         self.transforms = transforms
@@ -125,22 +166,28 @@ class ImageDataset(object):
         classes, idx_to_cls, cls_to_idx, class_folders = self.find_classes_()
         self.class_dataset = class_folders
 
+        if self.imagenet_val:
+            img2cls, synsets = get_img2cls_mapping(self.root)
+            self.classes = synsets
+
         if self.class_dataset:
             if self.file_names:
                 _, cls_to_files = get_classes(self.file_names)
-                self.samples = class_dataset(self.root, out_path, cls_to_idx, self.things, add_ref_imgs, cls_to_files)
+                self.samples = class_dataset(self.root, out_path, cls_to_idx, self.imagenet_train, self.things, add_ref_imgs, cls_to_files)
             else:
-                self.samples = class_dataset(self.root, out_path, cls_to_idx, self.things, add_ref_imgs)
+                self.samples = class_dataset(self.root, out_path, cls_to_idx, self.imagenet_train, self.things, add_ref_imgs)
             self.classes = classes
         else:
-            self.samples = instance_dataset(self.root, out_path, classes)
-            self.classes = list(cls_to_idx.keys())
+            self.samples = instance_dataset(self.root, out_path, classes, self.imagenet_val, img2cls if self.imagenet_val else None)
+            if not self.imagenet_val:
+                self.classes = list(cls_to_idx.keys())
 
         images, targets = zip(*self.samples)
 
         self.idx_to_cls = idx_to_cls
         self.cls_to_idx = cls_to_idx
         self.targets = targets
+        self.images = images
 
     def find_classes_(self) -> Tuple[list, dict, dict]:
         children = sorted([d.name for d in os.scandir(self.root) if d.is_dir()])
@@ -158,9 +205,16 @@ class ImageDataset(object):
                         raise FileNotFoundError(f'To extract features for THINGS images, concept file is required. Move {concept_file} to {data_path}.')
 
                     concept_ids = pd.read_csv(pjoin(data_path, concept_file), encoding='utf-8', sep='\t').uniqueID.tolist()
-                    assert len(classes) == len(concept_ids), '\nNumber of categories in dataset must be equal to the number of concept IDs. Check img folder.\n'
-                    classes = classes if classes == concept_ids else concept_ids
-            idx_to_cls = dict(enumerate(classes))
+                    assert len(children) == len(concept_ids), '\nNumber of categories in dataset must be equal to the number of concept IDs. Check img folder.\n'
+                    classes = children if children == concept_ids else concept_ids
+                else:
+                    classes = children
+            if self.imagenet_train:
+                cls_to_idx = get_syn2cls_mapping(self.root)
+                idx_to_cls = {idx:cls for cls, idx in cls_to_idx.items()}
+                classes = list(cls_to_idx.keys())
+            else:
+                idx_to_cls = dict(enumerate(classes))
         else:
             class_folders = False
             if self.things_behavior:
@@ -172,7 +226,8 @@ class ImageDataset(object):
                 else:
                     classes = sorted([f.name for f in os.scandir(self.root) if f.is_file() and parse_img_name(f.name)])
             idx_to_cls = dict(enumerate(list(map(rm_suffix, classes))))
-        cls_to_idx = {cls:idx for idx, cls in idx_to_cls.items()}
+        if not self.imagenet_train:
+            cls_to_idx = {cls:idx for idx, cls in idx_to_cls.items()}
         return classes, idx_to_cls, cls_to_idx, class_folders
 
     def __getitem__(self, idx:int) -> Tuple[Any, Any]:
